# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-19 00:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('storekit', '0002_auto_20171218_2351'),
    ]

    operations = [
        migrations.AlterField(
            model_name='inapp',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='purchase',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='receipt',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='response',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



from django.contrib import admin

from .models import InApp, Receipt, Response


class InAppAdmin(admin.ModelAdmin):
    list_display = (
        "quantity",
        "product_id",
        "transaction_id",
        "original_transaction_id",
        "purchase_date",
        "original_purchase_date",
        "is_trial_period",
    )
    list_display_links = (
        "quantity",
        "product_id",
        "transaction_id",
        "original_transaction_id",
        "purchase_date",
        "original_purchase_date",
    )


admin.site.register(InApp, InAppAdmin)


class ReceiptAdmin(admin.ModelAdmin):
    list_display = (
        "receipt_type",
        "adam_id",
        "app_item_id",
        "bundle_id",
        "application_version",
        "download_id",
        "version_external_identifier",
        "receipt_creation_date",
        "request_date",
        "original_purchase_date",
        "in_app",
    )
    list_display_links = (
        "receipt_type",
        "adam_id",
        "app_item_id",
        "bundle_id",
        "application_version",
        "download_id",
        "version_external_identifier",
        "receipt_creation_date",
        "request_date",
        "original_purchase_date",
        "in_app",
    )

    def get_queryset(self, request):
        return super().get_queryset(request).select_related("in_app")


admin.site.register(Receipt, ReceiptAdmin)


class ResponseAdmin(admin.ModelAdmin):
    list_display = ("status", "environment", "receipt")
    list_display_links = ("status", "environment", "receipt")

    def get_queryset(self, request):
        return super().get_queryset(request).select_realated("receipt")


admin.site.register(Response, ResponseAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-18 23:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('storekit', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='inapp',
            name='id',
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='purchase',
            name='id',
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='purchase',
            name='product_id',
            field=models.CharField(default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='receipt',
            name='id',
            field=models.AutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='response',
            name='id',
            field=models.AutoField(primary_key=True, serialize=False),
        ),
    ]



from django.db import models

from .fields import PositiveBigIntegerField


class InApp(models.Model):
    quantity = models.IntegerField(blank=True)
    product_id = models.CharField(blank=False, default="", max_length=255)
    transaction_id = PositiveBigIntegerField(blank=True)
    original_transaction_id = PositiveBigIntegerField(blank=True)
    purchase_date = models.CharField(blank=False, default="", max_length=255)
    purchase_date_ms = models.IntegerField(blank=True)
    purchase_date_pst = models.CharField(blank=False, default="", max_length=255)
    original_purchase_date = models.CharField(blank=False, default="", max_length=255)
    original_purchase_date_ms = models.IntegerField(blank=True)
    original_purchase_date_pst = models.CharField(blank=False, default="", max_length=255)
    is_trial_period = models.BooleanField(blank=False, default=False)

    def __init__(
        self,
        quantity,
        product_id,
        transaction_id,
        original_transaction_id,
        purchase_date,
        purchase_date_ms,
        purchase_date_pst,
        original_purchase_date,
        original_purchase_date_ms,
        original_purchase_date_pst,
        is_trial_period,
    ):
        models.Model.__init__(self)
        self.quantity = quantity
        self.product_id = product_id
        self.transaction_id = transaction_id
        self.original_transaction_id = original_transaction_id
        self.purchase_date = purchase_date
        self.purchase_date_ms = purchase_date_ms
        self.purchase_date_pst = purchase_date_pst
        self.original_purchase_date = original_purchase_date
        self.original_purchase_date_ms = original_purchase_date_ms
        self.original_purchase_date_pst = original_purchase_date_pst
        self.is_trial_period = is_trial_period

    def __str__(self):
        return "{}: {}".format(self.product_id, self.quantity)

    def __repr__(self):
        return '{} <ID: {}>: "{}"'.format(self.product_id, self.id, self.__class__.__name__)

    @classmethod
    def parser(cls, json, is_save=True):
        in_app = {
            "quantity": int(json["quantity"]),
            "product_id": json["product_id"],
            "transaction_id": int(json["transaction_id"]),
            "original_transaction_id": int(json["original_transaction_id"]),
            "purchase_date": json["purchase_date"],
            "purchase_date_ms": int(json["purchase_date_ms"]),
            "purchase_date_pst": json["purchase_date_pst"],
            "original_purchase_date": json["original_purchase_date"],
            "original_purchase_date_ms": int(json["original_purchase_date_ms"]),
            "original_purchase_date_pst": json["original_purchase_date_pst"],
            "is_trial_period": True if "true" == json["is_trial_period"] else False,
        }
        obj = cls(**in_app)
        if is_save:
            obj.save()
        return obj


class Receipt(models.Model):
    receipt_type = models.CharField(blank=False, default="", max_length=255)
    adam_id = models.IntegerField(blank=True)
    app_item_id = models.IntegerField(blank=True)
    bundle_id = models.CharField(blank=False, default="", max_length=255)
    application_version = models.CharField(blank=False, default="", max_length=255)
    download_id = models.IntegerField(blank=True)
    version_external_identifier = models.IntegerField(blank=True)
    receipt_creation_date = models.CharField(blank=False, default="", max_length=255)
    receipt_creation_date_ms = models.IntegerField(blank=True)
    receipt_creation_date_pst = models.CharField(blank=False, default="", max_length=255)
    request_date = models.CharField(blank=False, default="", max_length=255)
    request_date_ms = models.IntegerField(blank=True)
    request_date_pst = models.CharField(blank=False, default="", max_length=255)
    original_purchase_date = models.CharField(blank=False, default="", max_length=255)
    original_purchase_date_ms = models.IntegerField(blank=True)
    original_purchase_date_pst = models.CharField(blank=False, default="", max_length=255)
    original_application_version = models.CharField(blank=False, default="", max_length=255)
    in_app = models.ForeignKey(InApp, blank=True, on_delete=models.CASCADE)

    def __init__(
        self,
        receipt_type,
        adam_id,
        app_item_id,
        bundle_id,
        application_version,
        download_id,
        version_external_identifier,
        receipt_creation_date,
        receipt_creation_date_ms,
        receipt_creation_date_pst,
        request_date,
        request_date_ms,
        request_date_pst,
        original_purchase_date,
        original_purchase_date_ms,
        original_purchase_date_pst,
        original_application_version,
        in_app,
    ):
        models.Model.__init__(self)
        self.receipt_type = receipt_type
        self.adam_id = adam_id
        self.app_item_id = app_item_id
        self.bundle_id = bundle_id
        self.application_version = application_version
        self.download_id = download_id
        self.version_external_identifier = version_external_identifier
        self.receipt_creation_date = receipt_creation_date
        self.receipt_creation_date_ms = receipt_creation_date_ms
        self.receipt_creation_date_pst = receipt_creation_date_pst
        self.request_date = request_date
        self.request_date_ms = request_date_ms
        self.request_date_pst = request_date_pst
        self.original_purchase_date = original_purchase_date
        self.original_purchase_date_ms = original_purchase_date_ms
        self.original_purchase_date_pst = original_purchase_date_pst
        self.original_application_version = original_application_version
        self.in_app = in_app

    def __str__(self):
        return "[{}] {}: {}".format(self.receipt_type, self.bundle_id, self.application_version)

    def __repr__(self):
        return '<ID: {}>: "{}"'.format(self.id, self.__class__.__name__)

    @classmethod
    def parser(cls, json, in_app, is_save=True):
        receipt = {
            "receipt_type": json["receipt_type"],
            "adam_id": int(json["adam_id"]),
            "app_item_id": int(json["app_item_id"]),
            "bundle_id": json["bundle_id"],
            "application_version": json["application_version"],
            "download_id": int(json["download_id"]),
            "version_external_identifier": int(json["version_external_identifier"]),
            "receipt_creation_date": json["receipt_creation_date"],
            "receipt_creation_date_ms": int(json["receipt_creation_date_ms"]),
            "receipt_creation_date_pst": json["receipt_creation_date_pst"],
            "request_date": json["request_date"],
            "request_date_ms": int(json["request_date_ms"]),
            "request_date_pst": json["request_date_pst"],
            "original_purchase_date": json["original_purchase_date"],
            "original_purchase_date_ms": int(json["original_purchase_date_ms"]),
            "original_purchase_date_pst": json["original_purchase_date_pst"],
            "original_application_version": json["original_application_version"],
            "in_app": InApp.parser(in_app),
        }
        obj = cls(**receipt)
        if is_save:
            obj.save()
        return obj


class Response(models.Model):
    status = models.IntegerField(blank=True)
    environment = models.CharField(blank=False, default="", max_length=255)
    receipt = models.ForeignKey(Receipt, blank=True, on_delete=models.CASCADE)

    def __init__(self, status, environment, receipt):
        models.Model.__init__(self)
        self.status = status
        self.environment = environment
        self.receipt = receipt

    def __str__(self):
        return "Response: " + self.environment

    def __repr__(self):
        return '<ID: {}>: "{}"'.format(self.id, self.__class__.__name__)

    @classmethod
    def parser(cls, json, in_app, is_save=True):
        response = {
            "status": json["status"],
            "environment": json["environment"],
            "receipt": Receipt.parser(json["receipt"], in_app),
        }
        obj = cls(**response)
        if is_save:
            obj.save()
        return obj



from django.conf import settings

from .appstore import AppStoreValidator


def normal_receipt(receipt, sandbox):
    try:
        storekit_bundle_id = settings.STOREKIT_APP_BUNDLE_ID
    except AttributeError:
        raise AttributeError('Please set "STOREKIT_APP_BUNDLE_ID" in your settings.py')

    validator = AppStoreValidator(bundle_id=storekit_bundle_id, sandbox=sandbox)

    return validator.validate(receipt=receipt)


def subscribe_receipt(receipt, sandbox):
    try:
        storekit_app_bundle_id = settings.STOREKIT_APP_BUNDLE_ID
    except AttributeError:
        raise AttributeError('Please set "STOREKIT_APP_BUNDLE_ID" in your settings.py')

    try:
        storekit_purchased_secret = settings.STOREKIT_PURCHASED_SECRET
    except AttributeError:
        raise AttributeError('Please set "STOREKIT_PURCHASED_SECRET" in your settings.py')

    validator = AppStoreValidator(bundle_id=storekit_app_bundle_id, sandbox=sandbox)

    return validator.validate(receipt=receipt, password=storekit_purchased_secret)



import requests

from .errors import AppValidationError
from .models import Response

api_result_ok = 0
api_result_errors = {
    21000: "Bad json",
    21002: "Bad data",
    21003: "Receipt authentication",
    21004: "Shared secret mismatch",
    21005: "Server is unavailable",
    21006: "Subscription has expired",
    21007: "Sandbox receipt was sent to the production env",
    21008: "Production receipt was sent to the sandbox env",
}


class AppStoreValidator(object):
    def __init__(self, bundle_id, sandbox=False):
        self.bundle_id = bundle_id

        if sandbox:
            self.url = "https://sandbox.itunes.apple.com/verifyReceipt"
        else:
            self.url = "https://buy.itunes.apple.com/verifyReceipt"

    def validate(self, receipt, password=None):
        receipt_json = {"receipt-data": receipt}
        if password:
            receipt_json["password"] = password

        api_response = requests.post(self.url, json=receipt_json).json()

        status = api_response["status"]

        if status != api_result_ok:
            error = AppValidationError(api_result_errors.get(status, "Unknown API status"), api_response)
            raise error

        purchases = self._parse_receipt(api_response)
        return purchases

    def _parse_receipt(self, api_response):
        if self.bundle_id != api_response["receipt"]["bundle_id"]:
            raise AppValidationError("Bundle ID mismatch", api_response)
        return [Response.parser(api_response, in_app) for in_app in api_response["receipt"]["in_app"]]



# Generated by Django 2.0.4 on 2018-06-09 16:54

from django.db import migrations

import storekit.fields


class Migration(migrations.Migration):

    dependencies = [
        ('storekit', '0004_auto_20180605_1629'),
    ]

    operations = [
        migrations.AlterField(
            model_name='inapp',
            name='original_transaction_id',
            field=storekit.fields.PositiveBigIntegerField(blank=True),
        ),
        migrations.AlterField(
            model_name='inapp',
            name='transaction_id',
            field=storekit.fields.PositiveBigIntegerField(blank=True),
        ),
    ]



# Generated by Django 2.0.4 on 2018-06-05 16:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('storekit', '0003_auto_20171219_0021'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='purchase',
            name='response',
        ),
        migrations.DeleteModel(
            name='Purchase',
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['storekit', 'storekit.migrations']

package_data = \
{'': ['*']}

install_requires = \
['Django>=2,<3', 'requests>=2,<3']

setup_kwargs = {
    'name': 'django-ios-storekit',
    'version': '1.0.8',
    'description': "iOS In-App Purchase's receipt validation server plugin for Django",
    'long_description': "# django-ios-storekit\n\n[![Tests](https://github.com/nnsnodnb/django-ios-storekit/actions/workflows/tests.yml/badge.svg?branch=master)](https://github.com/nnsnodnb/django-ios-storekit/actions/workflows/tests.yml)\n[![Linter](https://github.com/nnsnodnb/django-ios-storekit/actions/workflows/linter.yml/badge.svg?branch=master)](https://github.com/nnsnodnb/django-ios-storekit/actions/workflows/linter.yml)\n[![Coverage Status](https://coveralls.io/repos/github/nnsnodnb/django-ios-storekit/badge.svg?branch=master)](https://coveralls.io/github/nnsnodnb/django-ios-storekit?branch=master)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/django-ios-storekit)\n![PyPI](https://img.shields.io/pypi/v/django-ios-storekit)\n![PyPI - Format](https://img.shields.io/pypi/format/django-ios-storekit)\n![PyPI - Wheel](https://img.shields.io/pypi/wheel/django-ios-storekit)\n\nA Django plugin for iOS StoreKit server.\n\n## Supported python versions\n\n3.6.x ~ 3.8.x\n\n## Supported django versions\n\n2.x\n\n## Installation\n\n```shell script\n$ pip install django-ios-storekit\n```\n\nAdd `storekit` into `INSTALLED_APPS` in `settings.py` file.\n\n```python\nINSTALLED_APPS += (\n    'storekit',\n)\n```\n\n```shell script\n$ python manage.py migrate\n```\n\n## License\n\nThis software is licensed under the MIT License (See [LICENSE](https://github.com/nnsnodnb/django-ios-storekit/blob/master/LICENSE)).\n",
    'author': 'Yuya Oka',
    'author_email': 'nnsnodnb@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nnsnodnb/django-ios-storekit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from django.db.models import BigIntegerField
from django.utils.translation import gettext_lazy as _


class PositiveBigIntegerField(BigIntegerField):

    empty_strings_allowed = False
    description = _("Big (8 byte) positive integer")

    def db_type(self, connection):
        return "bigint UNSIGNED"

    def get_internal_type(self):
        return "PositiveSmallIntegerField"

    def formfield(self, **kwargs):
        defaults = {"min_value": 0, "max_value": BigIntegerField.MAX_BIGINT}
        defaults.update(kwargs)
        return super(PositiveBigIntegerField, self).formfield(**defaults)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-12-08 22:38
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='InApp',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('quantity', models.IntegerField(blank=True)),
                ('product_id', models.CharField(default='', max_length=255)),
                ('transaction_id', models.IntegerField(blank=True)),
                ('original_transaction_id', models.IntegerField(blank=True)),
                ('purchase_date', models.CharField(default='', max_length=255)),
                ('purchase_date_ms', models.IntegerField(blank=True)),
                ('purchase_date_pst', models.CharField(default='', max_length=255)),
                ('original_purchase_date', models.CharField(default='', max_length=255)),
                ('original_purchase_date_ms', models.IntegerField(blank=True)),
                ('original_purchase_date_pst', models.CharField(default='', max_length=255)),
                ('is_trial_period', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Purchase',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('transaction_id', models.IntegerField(blank=True)),
                ('product_id', models.IntegerField(blank=True)),
                ('quantity', models.IntegerField(default=0)),
                ('purchased_at', models.CharField(default='', max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Receipt',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('receipt_type', models.CharField(default='', max_length=255)),
                ('adam_id', models.IntegerField(blank=True)),
                ('app_item_id', models.IntegerField(blank=True)),
                ('bundle_id', models.CharField(default='', max_length=255)),
                ('application_version', models.CharField(default='', max_length=255)),
                ('download_id', models.IntegerField(blank=True)),
                ('version_external_identifier', models.IntegerField(blank=True)),
                ('receipt_creation_date', models.CharField(default='', max_length=255)),
                ('receipt_creation_date_ms', models.IntegerField(blank=True)),
                ('receipt_creation_date_pst', models.CharField(default='', max_length=255)),
                ('request_date', models.CharField(default='', max_length=255)),
                ('request_date_ms', models.IntegerField(blank=True)),
                ('request_date_pst', models.CharField(default='', max_length=255)),
                ('original_purchase_date', models.CharField(default='', max_length=255)),
                ('original_purchase_date_ms', models.IntegerField(blank=True)),
                ('original_purchase_date_pst', models.CharField(default='', max_length=255)),
                ('original_application_version', models.CharField(default='', max_length=255)),
                ('in_app', models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='storekit.InApp')),
            ],
        ),
        migrations.CreateModel(
            name='Response',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.IntegerField(blank=True)),
                ('environment', models.CharField(default='', max_length=255)),
                ('receipt', models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='storekit.Receipt')),
            ],
        ),
        migrations.AddField(
            model_name='purchase',
            name='response',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='storekit.Response'),
        ),
    ]
